#include "stdinc.hpp"


	Timer::Timer( const int _averagingWindow )
		: fail( false ), freq( 0 ), lastTime( 0 ), average( 0 ),
		averagingWindow( _averagingWindow ),
		times( new float[ _averagingWindow ] ),
		timeIndex( 0 ),
		numSamples( 0 )
	{
		{
			LARGE_INTEGER temp;
			
			if( QueryPerformanceFrequency( &temp ) == 0 )
				fail= true;
			else
			{
				freq= temp.QuadPart;
				
				QueryPerformanceCounter( &temp );
				
				lastTime= temp.QuadPart;
			}
		}
		
		for( int i= 0; i< averagingWindow; i++ )
		{
			times[i]= 0;
		}
	}



	Timer::~Timer()
	{
		delete []times;
	}



	bool Timer::Fail()
	{
		return fail;
	}



	float Timer::Tick()
	{
		LARGE_INTEGER temp;
		LONGLONG currTime;
		float returnValue;
		
		QueryPerformanceCounter( &temp );
		currTime= temp.QuadPart;
		
		returnValue= float((double)(currTime-(LONGLONG)lastTime)/
			(double)(LONGLONG)freq);
		
		lastTime= currTime;
		
		times[ timeIndex ]= returnValue;
		average+= times[ timeIndex ];
		
		timeIndex= (timeIndex+1)%averagingWindow;
		average-= times[ timeIndex ];

		numSamples++;
		
		return returnValue;
	}



	float Timer::IntervalAverage()
	{
		int divis= averagingWindow;

		if( numSamples < averagingWindow )
		{
			divis= numSamples;
		}

		return average/(float)divis;
	}
